# Helper module for Stanford IDG Perl coding style tests.
#
# This module is a wrapper around Test::Perl::Critic that sets it up to use
# the IDG perlcritic profile and perltidy configuration.  It provides
# idg_critic_ok and all_idg_critic_ok interfaces that wrap the corresponding
# Test::Perl::Critic interfaces.  (It can't provide them with the same name
# due to the unconditional export in Test::Perl::Critic.)
#
# Written by Russ Allbery <rra@stanford.edu>
# Copyright 2013
#     The Board of Trustees of the Leland Stanford Junior University
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

package Test::Stanford::IDG::Critic;

use 5.010;
use strict;
use warnings;

use Exporter qw(import);
use File::ShareDir qw(module_file);
use Test::Perl::Critic;

# Declare variables that should be set in BEGIN for robustness.
our (@EXPORT_OK, $VERSION);

# Set $VERSION and everything export-related in a BEGIN block for robustness
# against circular module loading.
BEGIN {
    @EXPORT_OK = qw(all_idg_critic_ok idg_critic_ok);
    $VERSION   = '1.01';
}

# Installed by the stanford-idg-perlcritic package.
our $PERLCRITICRC = module_file('Test::Stanford::IDG::Critic', 'perlcriticrc');
our $PERLTIDYRC   = module_file('Test::Stanford::IDG::Critic', 'perltidyrc');

# Older versions of Perl::Critic need a different version of perlcriticrc that
# also suppresses Miscellanea::RequireRcsKeywords.  We can't unconditionally
# suppress it since Perl::Critic warns about unknown policies in its profile.
if (defined($Perl::Critic::Policy::Miscellanea::RequireRcsKeywords::VERSION)) {
    $PERLCRITICRC .= '-norcs';
}

# Point Test::Perl::Critic at the appropriate configuration.
Test::Perl::Critic->import(-profile => $PERLCRITICRC);

# Wrapper around all_critic_ok that sets the appropriate perltidy
# configuration.
#
# @files - List of files or directories to check (possibly empty)
#
# Returns: Return status of all_critic_ok
sub all_idg_critic_ok {
    my (@paths) = @_;
    local $ENV{PERLTIDY} = $PERLTIDYRC;
    return all_critic_ok(@paths);
}

# Wrapper around critic_ok that sets the appropriate perltidy configuration.
#
# $file - File to check
# $name - Name of the test for test output (optional)
#
# Returns: Return status of critic_ok
sub idg_critic_ok {
    my ($file, $name) = @_;
    local $ENV{PERLTIDY} = $PERLTIDYRC;
    return critic_ok($file, $name);
}

1;
__END__

=for stopwords
Allbery IDG perlcritic perltidy PATHs

=head1 NAME

Test::Stanford::IDG::Critic - Call Test::Perl::Critic for Stanford IDG style

=head1 SYNOPSIS

    use Test::Stanford::IDG::Critic qw(all_idg_critic_ok);
    all_idg_critic_ok();

=head1 DESCRIPTION

This module wraps Test::Perl::Critic to point it to the Stanford IDG
profile and B<perltidy> configuration.  It provides idg_critic_ok() and
all_idg_critic_ok() functions that parallel the critic_ok() and
all_critic_ok() functions of Test::Perl::Critic.

=head1 FUNCTIONS

=over 4

=item idg_critic_ok(FILE [, NAME])

Tests FILE with Perl::Critic and reports the result as a TAP test.
Assumes that a TAP plan has already been output.  The optional NAME
argument will be used as the name of the test, if given.

=item all_idg_critic_ok([ PATH ... ])

Runs idg_critic_ok() for all Perl files beneath the list of PATHs, which
can be either directory names or file names.  If the list of PATHs is
empty, this function tries to find Perl files under F<blib> and, failing
that, B<lib>.  This function emits its own TAP plan, so you don't need
to specify the expected number of tests.

=back

=head1 FILES

=over 4

=item F</usr/share/stanford-idg-perlcritic/perlcriticrc>

The default B<perlcritic> profile used by this script.

=item F</usr/share/stanford-idg-perlcritic/perltidyrc>

The default B<perltidy> configuration used by this script.

=back

=head1 AUTHOR

Russ Allbery <rra@stanford.edu>

=cut
